//******************************************************************************
// dmx_sende_daten.c
//******************************************************************************
//  Autor          Datum       Version       Änderung
//  boigk.net      12.02.2020   0.1          Start
//  boigk.net      13.02.2020   0.2          Absolute Pfade für Dateiaufrufe
//                                           Verwendet. Leider funktioniert es
//                                           anders. Damit ist die Position der
//                                           DMX Daten und die Position der
//                                           Fehler Datei fest gesetzt auf:
//                                           /var/www/html/Config/
//******************************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include "/usr/include/asm-generic/termbits.h"
#include "/usr/include/asm-generic/ioctls.h"

#define ANZAHL_BYTES 518
#define DATEN_OFFSET 5

void vSchreibProzess (void);
void vFehlerMeldeFunktion(char *pcMeldung);

//************************************************************************/
//* main-Funktion
//************************************************************************/
int main()
{
  int i;
  int iReturn;
  int pid; 
  pid=fork();

  if(pid == -1)
  {
    //Fehler. Fork nicht möglich
    vFehlerMeldeFunktion("Schreibprozess konnte nicht angelegt werden");
  }
  else if(pid == 0)
  {
    //Lese Daten Prozess
    printf("Erzeuge Schreibprozess\n");
    vSchreibProzess();
    exit(0);
  }

for(;;)
  {
    sleep(10);
  }
}


//***************************************************
//
//***************************************************
//************************************************************************/
//* Schreibprozess
//************************************************************************/
void vSchreibProzess(void){
  //***************************************************
  // Variablen
  //***************************************************
  int intSendBytes,intReadBytes; //Rückgabeparameter von Funktionen
  int iPortFD;                   //Datei Handle für Serielle Schnittstelle
  struct termios2 config;        //Konfiguration für Serielle Schnittstelle
  int iReturn;                   //Rückgabewert Konfiguration
  char Buffer[ANZAHL_BYTES];     //Puffer für Daten
  int i;                         //Zählvariable für Schleife
  FILE *fDaten_Datei;            //Datei Handle für DMX Werte

  //***************************************************
  // Code
  //***************************************************
  //***************************************************
  // Init Buffer;
  //***************************************************
  for(i=0;i<ANZAHL_BYTES;i++)
  {
    Buffer[i]=0;
  }
  Buffer[0]=0x7E; //Startbyte
  Buffer[1]=0x06; //Protokollkennung
  Buffer[2]=0x01; //Anzahl Bytes niederwertiger Teil
  Buffer[3]=0x02; //Anzahl Bytes höherwertiger Teil
  Buffer[4]=0x00; //???
  Buffer[ANZAHL_BYTES-1]=0xE7; //Endbyte

  //***************************************************
  // Öffne Schnittstellendatei
  //***************************************************
  iPortFD = open("/dev/ttyUSB0",O_RDWR | O_NOCTTY);
  if (iPortFD <= 0)
  {
    vFehlerMeldeFunktion("Kann Serielle Schnittstelle nicht öffnen");
  }

  //***************************************************
  // Erstelle Konfigurationsstruktur
  //***************************************************
  ioctl(iPortFD, TCGETS2, config);
  config.c_cflag &= ~CBAUD;
  config.c_cflag |= BOTHER;
  config.c_ispeed = 250000;
  config.c_ospeed = 250000;

{
    //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    //Noch Offen. Warum ist das hier?
    //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    int speed;
    //speed = cfgetispeed(&attribs);
    printf("input speed: %lu\n", (unsigned long) speed);
    
 }

  //***************************************************
  // Schreibe Konfiguration
  //***************************************************
  iReturn = ioctl(iPortFD,TCSETS2, &config);
  if(iReturn != 0)
  {
     printf("Fehler ioctl: %x",iReturn);
     vFehlerMeldeFunktion("Fehler ioctl Zugriff");
  }
 
  //***************************************************
  // Schleife für Ausführung
  //***************************************************
  for(;;)
  {

      //***************************************************
      // Hole Daten vom Dateisystem
      //***************************************************
      fDaten_Datei = fopen("/var/www/html/Config/SendeDaten.dmx","r");
      if(fDaten_Datei == 0)
      {
         vFehlerMeldeFunktion("SendeDaten.dmx fehlt!");
      }
      else
      {
        fread(Buffer,ANZAHL_BYTES,1,fDaten_Datei);
        //***************************************************
        // Schließe Datei
        //***************************************************
        fclose(fDaten_Datei);
      }

  //***************************************************
  // Sende Daten
  //***************************************************
    intSendBytes = write (iPortFD, Buffer, ANZAHL_BYTES);
    if(intSendBytes != ANZAHL_BYTES)
    {
      printf("Daten konnten nicht geschrieben werden. Rückgabe: %x \n",intSendBytes);
      vFehlerMeldeFunktion("SendeDaten.dmx konnten nicht geschrieben werden.");
    }

    usleep(20);
  }


return;
}



//************************************************************************/
//* Fehler Melde Funktion
//************************************************************************/
void vFehlerMeldeFunktion(char *pcMeldung)
{
  //***************************************************
  // Variablen
  //***************************************************
  FILE *fFehler_Datei;     //Datei mit Fehlermeldungen
  struct stat Datei_Eigenschaften;

  #define FEHLER_DATEI_PFAD "/var/www/html/Config/Fehler.log"

  //***************************************************
  // Code
  //***************************************************
  fFehler_Datei = fopen(FEHLER_DATEI_PFAD, "a");
  stat(FEHLER_DATEI_PFAD,&Datei_Eigenschaften);
  printf("Datei Größe %d \n",Datei_Eigenschaften.st_size);

  //Schreibe nur wenn Datei kleiner als 60000 Byte ist.
  //Also ca. 60kb.

  if(Datei_Eigenschaften.st_size<60000)
  {
    fprintf(fFehler_Datei,pcMeldung);
    fprintf(fFehler_Datei,"<br> \n");
  }
  fclose(fFehler_Datei);

  return;
}

